"""
The aurori project

Copyright (C) 2022  Marcus Drobisch,

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

__authors__ = ["Marcus Drobisch"]
__contact__ = "aurori@fabba.space"
__credits__ = []
__license__ = "AGPLv3+"

from aurori.logs import logManager
import time
import threading


class NodeManager(object):
    """ The NodeManager holds all available node-types and load them while creation.
    """
    def __init__(self):
        self.node_classes = {}
        self.nodes = {}

    def register_node(self, name, fingerprint, identifier, version):
        logManager.info("Registered node")
        node_instance = None
        for k, v in self.node_classes.items():
            if v.identifier == identifier:
                node_instance = self.node_classes[k]()

        if node_instance is not None:
            self.nodes[fingerprint] = node_instance
        else:
            logManager.error(
                f"NodeManager is unable to register node {name} ({fingerprint}) with unknown identifier '{identifier}'"
            )

    def register_node_class(self, workspace, node_class):
        node_instance = node_class()
        nodekey = str(workspace.name) + '/' + node_instance.name
        self.node_classes[nodekey] = node_class

    def get_node_classes(self):
        return self.node_classes

    def statemachine(self):
        while True:
            time.sleep(2)
            logManager.info("run statemachine cylce {} {}".format(
                self.nodeSource,
                threading.current_thread().name))

    def init_manager(self, app, db, workspaceManager):
        self.app = app
        self.db = db
        self.workspaceManager = workspaceManager
        logManager.info("NodeManager initialized")

        from aurori.nodes.models import Node
        self.node = Node
        self.worker = threading.Thread(target=self.statemachine,
                                       name='NodeManagr')
        self.worker.setDaemon(True)
        # self.worker.start()



"""
The aurori project

Copyright (C) 2022  Marcus Drobisch,

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

__authors__ = ["Marcus Drobisch"]
__contact__ = "aurori@fabba.space"
__credits__ = []
__license__ = "AGPLv3+"

from aurori.actions.actionGenerator import BaseAction


class ResetLoginAction(BaseAction):
    def __init__(self, app):
        print("Instance of ResetLoginAction created")


class UpdateSessionTokenAction(BaseAction):
    action = 'updateToken'
    target = 'webclient'

    def __init__(self, app):
        pass

    @classmethod
    def generate(cls, token, userinfo, delay=0.0):
        action = super(UpdateSessionTokenAction, cls).generate(delay)
        action['token'] = token
        action['userinfo'] = userinfo

        return action


class RouteAction(BaseAction):
    action = 'route'
    target = 'webclient'

    def __init__(self, app):
        pass

    @classmethod
    def generate(cls, route, delay=0.0):
        action = super(RouteAction, cls).generate(delay)
        action['route'] = route
        return action


class NotificationAction(BaseAction):
    action = 'notify'
    target = 'webclient'

    def __init__(self, app):
        pass

    @classmethod
    def generate(cls, message, messagetype, delay=0.0):
        action = super(NotificationAction, cls).generate(delay)
        action['message'] = message
        action['messagetype'] = messagetype
        return action


class LoadViewAction(BaseAction):
    action = 'loadView'
    target = 'webclient'

    def __init__(self, app):
        pass

    @classmethod
    def generate(cls, uri, properties, entries, metadata={}, delay=0.0):
        action = super(LoadViewAction, cls).generate(delay)
        action['viewname'] = uri
        action['viewdata'] = {}
        action['viewdata']['properties'] = properties
        action['viewdata']['entries'] = entries
        action['viewdata']['metadata'] = metadata
        return action


class UpdateMenuAction(BaseAction):
    action = 'updateMenu'
    target = 'webclient'

    def __init__(self, app):
        pass

    @classmethod
    def generate(cls, menu, delay=0.0):
        action = super(UpdateMenuAction, cls).generate(delay)
        action['data'] = menu
        return action


class UpdateActionlinkStatusAction(BaseAction):
    action = 'updateActionlinkStatus'
    target = 'webclient'

    def __init__(self, app):
        pass

    @classmethod
    def generate(cls, status, message, delay=0.0):
        action = super(UpdateActionlinkStatusAction, cls).generate(delay)
        action['status'] = status
        action['message'] = message
        return action



"""
The aurori project

Copyright (C) 2022  Marcus Drobisch,

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

__authors__ = ["Marcus Drobisch"]
__contact__ = "aurori@fabba.space"
__credits__ = []
__license__ = "AGPLv3+"

from aurori.logs import logs_bp, logManager


@logs_bp.route('/api/v1/log', methods=["GET"])
def log():
    log_string = ""
    for e in logManager.logQueue:
        log_string = log_string + e + "\n"
    return log_string



"""
The aurori project

Copyright (C) 2022  Marcus Drobisch,

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

__authors__ = ["Marcus Drobisch"]
__contact__ = "aurori@fabba.space"
__credits__ = []
__license__ = "AGPLv3+"

import logging
import sys
from os import environ

from flask import Flask, Blueprint
from flask_bcrypt import Bcrypt
from flask_jwt_extended import JWTManager
from flask_sqlalchemy import SQLAlchemy
from flask_migrate import Migrate
from flask_cors import CORS
from sqlalchemy_utils import database_exists

from aurori.config import configure_app, config_manager
from aurori.version import version
from aurori.logs import logManager
from aurori.workspaces import workspaceManager
from aurori.nodes import nodeManager
from aurori.users import userManager
from aurori.messages import messageManager
from aurori.monits import monit_manager
from aurori.jobs import jobManager
from aurori.actions import actionManager
from aurori.files import fileManager
from aurori.menus import menuBuilder
from aurori.openproject import openprojectManager
from aurori.widgets import widgetManager

jwt = JWTManager()
db = SQLAlchemy()
bcrypt = Bcrypt()
migrate = Migrate()

__version__ = version


def create_app(config, flask_config=None, minimal=False):
    config_manager.init_manager(config, flask_config)
    app = Flask(__name__, static_folder="./client")
    # configure the app
    configure_app(app, config, flask_config)

    if __name__ != '__main__':
        gunicorn_logger = logging.getLogger('gunicorn.error')
        app.logger.handlers = gunicorn_logger.handlers
        app.logger.setLevel(gunicorn_logger.level)
        if 'log_level' in config['SYSTEM']:
            app.logger.setLevel(
                logging._nameToLevel.get(config['SYSTEM']['log_level']))
    # logging.basicConfig(level=logging.DEBUG, format='%(threadName)s %(message)s')

    gettrace = getattr(sys, 'gettrace', None)
    if gettrace and gettrace():
        app.logger.setLevel(logging.DEBUG)

    if not minimal:
        CORS(app)
        logManager.init_app(app)
        logManager.info("Aurori framework started: " + version)
        jwt.init_app(app)
        workspaceManager.discover_models()
        bcrypt.init_app(app)

    from aurori import app_bp
    app.register_blueprint(app_bp)

    from aurori.actions import actions_bp, models  # noqa: F401, F811
    app.register_blueprint(actions_bp)

    from aurori.monits import monit_bp, models  # noqa: F401, F811
    app.register_blueprint(monit_bp)

    from aurori.users import auth_bp, basemodels  # noqa: F401, F811
    app.register_blueprint(auth_bp)

    from aurori.logs import logs_bp
    app.register_blueprint(logs_bp)

    from aurori.jobs import jobs_bp, models  # noqa: F401, F811
    app.register_blueprint(jobs_bp)

    from aurori.messages import messages_bp
    app.register_blueprint(messages_bp)

    from aurori.files import files_bp
    app.register_blueprint(files_bp)

    from aurori.nodes import nodes_bp
    app.register_blueprint(nodes_bp)

    from aurori.openproject import openproject_bp
    app.register_blueprint(openproject_bp)

    # import workspace blueprint and models
    from aurori.workspaces import workspaces_bp
    from aurori.workspaces import models

    app.register_blueprint(workspaces_bp)

    from aurori.api import api_bp
    app.register_blueprint(api_bp)

    from aurori.widgets import widgets_bp
    app.register_blueprint(widgets_bp)

    is_database_new = not database_exists(
        app.config['SQLALCHEMY_DATABASE_URI'])

    only_create_all_on_new_database_env = environ.get(
        'ONLY_CREATE_ALL_ON_NEW_DATABASE')
    if only_create_all_on_new_database_env is None:
        check_for_database_new = True
    else:
        if only_create_all_on_new_database_env == "1":
            check_for_database_new = True
        else:
            check_for_database_new = False

    db.init_app(app)

    # only create all schemas for new databases
    # otherwise this breaks the migration path
    if is_database_new or check_for_database_new is False:
        with app.app_context():
            db.create_all()

    if not minimal:
        migrate.init_app(app, db)
        jobManager.init_manager(app, db, config_manager.config)
        workspaceManager.init_app(app, db, config_manager.config)
        userManager.init_manager(app, db, workspaceManager,
                                 config_manager.config)
        monit_manager.init_manager(app, db, userManager, workspaceManager,
                                   config_manager.config)
        nodeManager.init_manager(app, db, workspaceManager)
        messageManager.init_manager(app, db, workspaceManager,
                                    config_manager.config)
        menuBuilder.init_builder(app, db, userManager, workspaceManager)
        actionManager.init_manager(app, db, userManager, menuBuilder,
                                   workspaceManager, nodeManager,
                                   config_manager.config)
        fileManager.init_manager(app, db, config_manager.config)
        openprojectManager.init_manager(app, db, openprojectManager,
                                        config_manager.config)
        widgetManager.init_manager(app, db, userManager, workspaceManager,
                                   config_manager.config)

    return app


# declare app routes
app_bp = Blueprint('app', __name__, static_folder="../client")
from aurori import routes



from aurori import db
from sqlalchemy_utils import ArrowType
import arrow
from aurori.users.basemodels import UserBase

class User(UserBase):
    global_user_id = db.Column(db.String(255), default='')
    home_server = db.Column(db.String(255), default='')



"""
The aurori project

Copyright (C) 2022  Marcus Drobisch,

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

__authors__ = ["Marcus Drobisch"]
__contact__ = "aurori@fabba.space"
__credits__ = []
__license__ = "AGPLv3+"

import requests
import math
from concurrent.futures import ThreadPoolExecutor
from requests.auth import HTTPBasicAuth
import json
from aurori.exceptions import NotFoundError


def patch_workpackage(base_url,
                      api_url_prefix,
                      key,
                      workpackage_id,
                      patch_dict,
                      notify=False):
    headers = {'content-type': 'application/json'}
    auth = HTTPBasicAuth('apikey', key)
    get_response = requests.get(base_url + api_url_prefix +
                                '/api/v3/work_packages/' + str(workpackage_id),
                                auth=auth).content
    get_response = json.loads(get_response)

    if get_response[
            'errorIdentifier'] == 'urn:openproject-org:api:v3:errors:NotFound':
        raise NotFoundError(
            f"Patching /api/v3/work_packages/{str(workpackage_id)} failed")

    if notify is True:
        notify_string = '?notify=true'
    else:
        notify_string = '?notify=false'

    patch_dict['lockVersion'] = get_response['lockVersion']
    patch_response = requests.patch(base_url + api_url_prefix +
                                    '/api/v3/work_packages/' +
                                    str(workpackage_id) + notify_string,
                                    data=json.dumps(patch_dict),
                                    headers=headers,
                                    auth=auth).content
    patch_response = json.loads(patch_response)
    pass


def getWorkpackagesOfProject(base_url,
                             api_url_prefix,
                             id,
                             key,
                             filter_string=None,
                             include_closed_packages=False):
    def get_partial_request(request, offset, auth):
        response = requests.get(request + '&offset=' + str(offset),
                                auth=auth).content
        response = json.loads(response)
        print(
            f"Got packages {offset} [{len( response['_embedded']['elements'] )}]"
        )

        return response['_embedded']['elements']

    auth = HTTPBasicAuth('apikey', key)
    if include_closed_packages is True:
        if filter_string is None:
            full_filter_string = '?filters=[ { "status_id": { "operator": "*", "values": null } }]&'
        else:
            full_filter_string = '?filters=[' + filter_string + ' , { "status_id": { "operator": "o", "values": null } }]&'
    else:
        if filter_string is None:
            full_filter_string = '?'
        else:
            full_filter_string = '?filters=[' + filter_string + ']&'

    packages = []
    pagesize = 150
    base_request_string = base_url + api_url_prefix + '/api/v3/projects/' + str(
        id) + '/work_packages' + full_filter_string + 'pageSize=' + str(
            pagesize)

    first_response = requests.get(base_request_string + '&offset=1',
                                  auth=auth).content
    first_response = json.loads(first_response)
    packages.extend(first_response['_embedded']['elements'])

    pages = math.ceil(first_response['total'] / pagesize) + 1
    with ThreadPoolExecutor() as executor:
        partials = []
        for offset in range(2, pages):
            partials.append(
                executor.submit(get_partial_request, base_request_string,
                                offset, auth))
    for p in partials:
        packages.extend(p.result())
    return packages


def get_user_by_mail(base_url, api_url_prefix, key, email):
    auth = HTTPBasicAuth('apikey', key)
    response = requests.get(
        base_url + api_url_prefix + '/api/v3/users' + '?filters=[ ' +
        '{ "name": { "operator": "=", "values": ["' + email + '" ] } } ,' +
        '{ "status": { "operator": "=", "values": [ "active" ] } } ]',
        auth=auth).content
    response = json.loads(response)
    users = {}
    # return only the first
    for u in response['_embedded']['elements']:
        return u


def get_user_list(base_url, api_url_prefix, key):
    auth = HTTPBasicAuth('apikey', key)
    response = requests.get(base_url + api_url_prefix + '/api/v3/users' +
                            '?pageSize=500',
                            auth=auth).content
    response = json.loads(response)
    users = {}
    for u in response['_embedded']['elements']:
        users[int(u['id'])] = u

    return users


def get_workpackages_by_filter(base_url,
                               api_url_prefix,
                               key,
                               filter_string,
                               include_closed_packages=False):
    auth = HTTPBasicAuth('apikey', key)
    if include_closed_packages is True:
        full_filter_string = '?filters=[' + filter_string + ' , { "status_id": { "operator": "*", "values": null } }]'
    else:
        full_filter_string = '?filters=[' + filter_string + ']'

    packages = []
    offset = 1
    while True:
        response = requests.get(base_url + api_url_prefix +
                                '/api/v3/work_packages' + full_filter_string +
                                '&pageSize=500' + '&offset=' + str(offset),
                                auth=auth).content
        response = json.loads(response)

        packages.extend(response['_embedded']['elements'])
        if response['count'] == response['pageSize']:
            offset += 1
        else:
            break
    return packages


def get_responsible_of_workpackage(workpackage):
    if 'responsible' not in workpackage['_links']:
        return None
    else:
        if workpackage['_links']['responsible']['href'] is None:
            return None
        else:
            workpackage['_links']['responsible']['id'] = int(
                workpackage['_links']['responsible']['href'].split('/')[-1])
            return workpackage['_links']['responsible']


def get_assignee_of_workpackage(workpackage):
    if 'assignee' not in workpackage['_links']:
        return None
    else:
        if workpackage['_links']['assignee']['href'] is None:
            return None
        else:
            workpackage['_links']['assignee']['id'] = int(
                workpackage['_links']['assignee']['href'].split('/')[-1])
            return workpackage['_links']['assignee']


def get_parent_of_workpackage(workpackage):
    if 'parent' not in workpackage['_links']:
        return None
    else:
        if workpackage['_links']['parent']['href'] is None:
            return None
        else:
            workpackage['_links']['parent']['id'] = int(
                workpackage['_links']['parent']['href'].split('/')[-1])
            return workpackage['_links']['parent']


def get_children_of_workpackage(workpackage):
    if 'children' not in workpackage['_links']:
        return []
    else:
        return workpackage['_links']['children']


def get_relations_of_workpackage(base_url, api_url_prefix, key, workpackage):
    headers = {'content-type': 'application/json'}
    auth = HTTPBasicAuth('apikey', key)
    relation_relative_url = workpackage['_links']['relations']['href']
    request_url = base_url + relation_relative_url
    data = {}
    response = requests.get(request_url,
                            auth=auth,
                            data=json.dumps(data),
                            headers=headers)
    response = json.loads(response.content)
    if ('_embedded' in response) and ('elements' in response['_embedded']):
        print(f"Got {len(response['_embedded']['elements'])} relations for",
              workpackage['id'])
        return response['_embedded']['elements']
    else:
        print("Got no relations for", workpackage['id'])
        return []


def get_activities_of_workpackage(base_url, api_url_prefix, key, workpackage):
    headers = {'content-type': 'application/json'}
    auth = HTTPBasicAuth('apikey', key)
    relation_relative_url = workpackage['_links']['activities']['href']
    request_url = base_url + relation_relative_url
    data = {}
    response = requests.get(request_url,
                            auth=auth,
                            data=json.dumps(data),
                            headers=headers)
    response = json.loads(response.content)
    if ('_embedded' in response) and ('elements' in response['_embedded']):
        return response['_embedded']['elements']
    else:
        return []



from aurori.files.storage_backends.base import BaseStorage

import os


class Storage(BaseStorage):
    def store_file(self, file_storage, file, file_object):
        try:
            path = os.path.join(
                file_storage.path,
                file_object.workspace or 'generic',
                file_object.uuid_string[0:2],
                file_object.uuid_string[2:4],
                file_object.uuid_string,
            )

            os.makedirs(path, exist_ok=True)
            file.save(os.path.join(path, file_object.file_name))
            file_object.path = path
            file_object.mime_type = file.content_type
            file_object.size_in_bytes = os.path.getsize(
                os.path.join(path, file_object.file_name))
            return True
        except Exception:
            return False

    def load_file(self, file_object):
        try:
            return open(os.path.join(file_object.path, file_object.file_name),
                        'r+b').read()
        except Exception:
            return None

    def remove_file(self, file_object):
        try:
            file_path = os.path.join(file_object.path, file_object.file_name)
            if os.path.exists(file_path):
                os.remove(file_path)
                os.removedirs(file_object.path)
            return True
        except Exception:
            return False



"""
The aurori project

Copyright (C) 2022  Marcus Drobisch,

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""


import os
import json
import configparser
from ast import literal_eval

basedir = os.path.abspath(os.getcwd())


def remove_quotes(original):
    d = original.copy()
    for key, value in d.items():
        if isinstance(value, str):
            s = d[key]
            if s.startswith(('"', "'")):
                s = s[1:]
            if s.endswith(('"', "'")):
                s = s[:-1]
            d[key] = s
        if isinstance(value, dict):
            d[key] = remove_quotes(value)
    return d


class SystemConfigs:
    def __init__(self, preferences_ini):
        self.preferences_ini = preferences_ini

        self.config = configparser.ConfigParser()
        self.config.read(preferences_ini)

        self.d = self.to_dict(self.config._sections)

    def as_dict(self):
        return self.d

    def to_dict(self, config):
        """
        Nested OrderedDict to normal dict and remove quotes around string
        """
        d = json.loads(json.dumps(config))
        d = remove_quotes(d)
        return d


def load_config(path):
    def remove_quotes(data):
        d = data.copy()
        for key, value in d.items():
            if isinstance(value, str):
                s = d[key]
                if s.startswith(('"', "'")):
                    s = s[1:]
                    s = s[:-1]
                else:
                    s = literal_eval(s)
                d[key] = s
            if isinstance(value, dict):
                d[key] = remove_quotes(value)
        return d

    cfg_parser = configparser.ConfigParser()
    # add default sections
    cfg_parser.add_section('LDAP')
    cfg_parser.add_section('MAIL')
    cfg_parser.add_section('SYSTEM')
    # read and overwrite
    cfg_parser.read(path)
    data = json.loads(json.dumps(cfg_parser._sections))
    config = remove_quotes(data)
    return config


def configure_app(app, config, flask_config):
    # Configure application to store JWTs in cookies
    app.config['JWT_TOKEN_LOCATION'] = ['headers']
    app.config['CORS_HEADERS'] = 'Content-Type'

    # Only allow JWT cookies to be sent over https. In production, this
    # should likely be True
    app.config['JWT_COOKIE_SECURE'] = False

    # Set the cookie paths, so that you are only sending your access token
    # cookie to the access endpoints, and only sending your refresh token
    # to the refresh endpoint. Technically this is optional, but it is in
    # your best interest to not send additional cookies in the request if
    # they aren't needed.
    app.config['JWT_ACCESS_COOKIE_PATH'] = '/api/v1/'
    app.config['JWT_REFRESH_COOKIE_PATH'] = '/api/v1/'
    app.config['SQLALCHEMY_DATABASE_URI'] = os.environ.get('DATABASE_URL') \
            or 'sqlite:///' + os.path.join(basedir, 'app.db')

    # Enable csrf double submit protection. See this for a thorough
    # explanation: http://www.redotheweb.com/2015/11/09/api-security.html
    app.config['JWT_COOKIE_CSRF_PROTECT'] = False

    # Set the secret key to sign the JWTs with
    app.config['JWT_SECRET_KEY'] = 'super-secret'  # Change this!

    app.config['SEND_FILE_MAX_AGE_DEFAULT'] = 0

    app.config['SQLALCHEMY_TRACK_MODIFICATIONS'] = False

    app.config['MAIL_SERVER'] = config['MAIL'].get('server', 'server')
    app.config['MAIL_PORT'] = config['MAIL'].get('port', 465)
    app.config['MAIL_USERNAME'] = config['MAIL'].get('username', 'username')
    app.config['MAIL_PASSWORD'] = config['MAIL'].get('password', 'password')
    app.config['MAIL_USE_TLS'] = config['MAIL'].get('tls', False)
    app.config['MAIL_USE_SSL'] = config['MAIL'].get('ssl', True)
    app.config['MAIL_SENDER'] = config['MAIL'].get('sender', 'test@test.com')

    if flask_config:
        app.config.update(flask_config)
    # app.config.from_pyfile('config.cfg', silent=True) # instance-folders configuration
    print("Cofing loaded")


class ConfigManager(object):
    """ The MonitManager ...
    """
    def __init__(self):
        self.config = None

    def init_manager(self, config, flask_config):
        self.config = config
        self.flask_config = flask_config


class Config(dict):
    def __init__(self, config_path="config.ini"):
        super(Config, self).__init__()
        cfg = load_config(config_path)
        self.update(cfg)


config_manager = ConfigManager()



"""
The aurori project

Copyright (C) 2022  Marcus Drobisch,

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

__authors__ = ["Marcus Drobisch"]
__contact__ = "aurori@fabba.space"
__credits__ = []
__license__ = "AGPLv3+"

from aurori.nodes import nodes_bp
from aurori.logs import logManager
from flask import make_response, request
from flask_cors import cross_origin
import json


@cross_origin()
@nodes_bp.route('/api/v1/nodes', methods=["POST"])
def nodes():
    s = json.dumps(request.json, indent=4)
    logManager.info("node request from: {}:\n{}".format(
        request.remote_addr, s))
    return make_response("ready for test", 200)


@nodes_bp.after_request
def after_request(response):
    header = response.headers
    header['Access-Control-Allow-Origin'] = '*'
    header['Access-Control-Allow-Headers'] = '*'
    header['Access-Control-Allow-Methods'] = '*'
    return response



"""
The aurori project

Copyright (C) 2022  Marcus Drobisch,

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

__authors__ = ["Marcus Drobisch"]
__contact__ = "aurori@fabba.space"
__credits__ = []
__license__ = "AGPLv3+"

from enum import Enum


class MonitType(Enum):
    PROPERTIES_ONLY = "PROPERTIES_ONLY"  # all properties only (e.g. count, last, average, min, max)
    TIMELINE = "TIMELINE"  # timeline monit of events aggregated (see MonitPropertyType)
    BUFFER = "BUFFER"  # a fixed size buffer (circular buffer) containing the events


class MonitChartType(Enum):
    SCATTER = "SCATTER"
    LINE = "LINE"
    BAR = "BAR"
    NONE = "NONE"


class MonitTimeAggregationType(Enum):
    HOURLY = "HOURLY"  # aggregate events to hourly timespan
    DAILY = "DAILY"  # aggregate events to daily timespan
    MONTHLY = "MONTHLY"  # aggregate events to monthly timespan


class MonitTimeSpanDisplayType(Enum):
    DAY = "DAY"  # aggregate events to hourly timespan
    WEEK = "WEEK"  # aggregate events to daily timespan
    MONTH = "MONTH"  # aggregate events to monthly timespan
    UNSET = "UNSET"


class MonitPropertyType(Enum):
    COUNT = "COUNT"  # count the events in the aggr. timespan (ignore values)
    LAST_VALUE = "LAST VALUE"  # take the last value from all events in the aggr. timespan
    SUM = "SUM"  # sum the values of all events in the aggr. timespan
    AVERAGE = "AVERAGE"  # take the average of the values from all events in the aggr. timespan
    TOTAL_COUNT = "TOTAL COUNT"
    TOTAL_SUM = "TOTAL COUNT"
    UNSET = "UNSET"


class MonitViewStats(object):
    def __init__(self, last, min, max, average, sum, count, last_event_at,
                 last_reset_at):
        self.last = last
        self.min = min
        self.max = max
        self.average = average
        self.sum = sum
        self.count = count
        self.last_event_at = last_event_at
        self.last_reset_at = last_reset_at


"""
The aurori project

Copyright (C) 2022  Marcus Drobisch,

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

__authors__ = ["Marcus Drobisch"]
__contact__ = "aurori@fabba.space"
__credits__ = []
__license__ = "AGPLv3+"

from aurori.messages import messages_bp
from aurori.messages import messageManager
from flask_jwt_extended import jwt_required, jwt_optional
from jinja2 import Template


@messages_bp.route('/api/v1/message/<int:id>', methods=['GET'])
@jwt_optional
def message(id):

    message = messageManager.message.query.filter_by(id=id).first()

    # Get File Content in String
    jinja2_template_string = open(message.template_path, 'r').read()

    # Create Template Object
    template = Template(jinja2_template_string)

    content = template.render(**message.data)
    return content, 200


"""
The aurori project

Copyright (C) 2022  Marcus Drobisch,

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

__authors__ = ["Marcus Drobisch"]
__contact__ = "aurori@fabba.space"
__credits__ = []
__license__ = "AGPLv3+"

from aurori.monits.types import MonitChartType, MonitPropertyType, MonitTimeSpanDisplayType

ALL = ["*"]


class MonitView(object):
    def __init__(self,
                 name,
                 label,
                 chart_type: MonitChartType,
                 description="",
                 display_classifier=ALL,
                 display_property=MonitPropertyType.SUM,
                 display_timespan=MonitTimeSpanDisplayType.WEEK,
                 fill_timeline_gaps=False):
        self.name = name
        self.label = label
        self.description = description
        self.display_classifier = display_classifier
        self.display_property = display_property
        self.display_timespan = display_timespan
        self.chart_type = chart_type
        self.fill_timeline_gaps = fill_timeline_gaps



"""
The aurori project

Copyright (C) 2022  Marcus Drobisch,

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

__authors__ = ["Marcus Drobisch"]
__contact__ = "aurori@fabba.space"
__credits__ = []
__license__ = "AGPLv3+"

from functools import wraps
from flask import g, Response


def require_user_permission(permissions):
    def decorator(func):
        @wraps(func)
        def wrapper(*args, **kwargs):
            if g.user is None:
                return Response('Authentication required', 401, {
                    'WWW-Authenticate':
                    'Basic realm="Authentication required"'
                })
            for p in permissions:
                if g.user.has_permission(p):
                    return func(*args, **kwargs)
            return Response('Insufficient user permissions', 403)

        return wrapper

    return decorator


def require_user_authentification(f):
    @wraps(f)
    def decorator(*args, **kwargs):
        if g.user is None:
            return Response(
                'Authentication required', 401,
                {'WWW-Authenticate': 'Basic realm="Authentication required"'})
        return f(*args, **kwargs)

    return decorator



""" The ordenario app. """

from aurori.config import Config
from aurori import create_app, db
from environment import create_environment

config = Config(config_path="config.ini")
app = create_app(config)

print()
print(app.url_map)

with app.app_context():
    create_environment(app, db)

if __name__ == '__main__':
    app.run()


"""
The aurori project

Copyright (C) 2022  Marcus Drobisch,

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

__authors__ = ["Marcus Drobisch"]
__contact__ = "aurori@fabba.space"
__credits__ = []
__license__ = "AGPLv3+"

import os
import pathlib
import pkgutil
import inspect
from aurori.workspaces.workspace import Workspace
from aurori.workspaces.workspaceHooks import WorkspaceHooks
from aurori.logs import logManager
import sys
import traceback
import sqlalchemy as sa


class WorkspaceManager(object):
    """ The WorkspaceManager holds all available workspaces and load them while creation.
    """
    def __init__(self, workspaceSource):
        self.workspaceSource = workspaceSource
        self.app = None
        self.db = None
        self.config = None
        self.workspaces = None
        self.user_class = None

    def init_app(self, app, db, config):
        self.app = app
        self.db = db
        self.config = config
        with self.app.app_context():
            self.reloadWorkspaces()
            self.registerWorkspacePlugins()
            self.registerWorkspacePermissions()
            self.registerWorkspaceAPIs()
            logManager.info("Workspaces initialized")

    def getWorkspace(self, name):
        for w in self.workspaces:
            if w.name is name:
                return w
        return None

    def triggerWorkspaceHooks(self, hook: WorkspaceHooks, **kwargs):
        for w in self.workspaces:
            try:
                if hook == WorkspaceHooks.CREATEUSER:
                    w.createUserHook(**kwargs)
                if hook == WorkspaceHooks.REMOVEUSER:
                    w.removeUserHook(**kwargs)
            except Exception as e:
                logManager.error('Failed to run hook {} on {} with {}'.format(
                    hook, w.name, e))

    def reloadWorkspaces(self):
        """Reset the list of all plugins and initiate the walk over the main
        provided plugin package to load all available plugins
        """
        self.workspaces = []
        self.seen_paths = []
        logManager.info("")
        logManager.info(
            f'Discover workspaces in path : {self.workspaceSource}')
        self.discover_workspaces(self.workspaceSource)

    def registerWorkspacePermissions(self):
        """ Run createPermissions for all workspaces and store permissions
        """
        logManager.info("")
        all_permissions = {}
        for ws in self.workspaces:

            workspace_permissions = ws.permissions
            if workspace_permissions is not None:
                all_permissions = {**all_permissions, **workspace_permissions}
            logManager.info(
                f'Register permissions for {ws.name}-workspace : {workspace_permissions}'
            )

        logManager.info(f'Delete orphaned permissions')

        # delete orphaned permissions for security reasons
        from .models import Permission
        from aurori import db
        engine = db.get_engine()
        table_exists = sa.inspect(engine).has_table(Permission.__tablename__)
        if table_exists:
            db_permissions = Permission.query.all()
            for permission in db_permissions:
                if permission.name not in all_permissions:
                    print("Delete orphaned permission: ", permission)
                    db.session.delete(permission)
                    db.session.commit()

    def registerWorkspaceAPIs(self):
        from aurori.api.check import check_before_request

        logManager.info("")
        logManager.info("Register api's of workspaces:")
        logManager.info("")
        for w in self.workspaces:
            api_workspace_path = (self.workspaceSource) + '.' + w.name + '.api'
            try:
                api_module = __import__(api_workspace_path, fromlist=['blah'])
                module_members = inspect.getmembers(api_module)
                blueprint = None
                for _name, member in module_members:
                    if _name == 'workspace_bp':
                        blueprint = member
                if blueprint is not None:
                    logManager.info(f'Register "{w.name}" workspace API')
                    blueprint.before_request(check_before_request)
                    self.app.register_blueprint(
                        blueprint,
                        url_prefix='/api/v1/workspaces/' + str(w.name.lower()))
                    pass
            except ModuleNotFoundError:
                logManager.info(
                    f'No specific API found for "{w.name}" workspace')
        logManager.info("")

    def registerWorkspacePlugins(self):
        """Recursively walk the supplied package to retrieve components for all plugins (workspaces)
        """
        logManager.info("")
        logManager.info("Register components from workspaces:")
        logManager.info("")

        for w in self.workspaces:
            logManager.info(f'Workspace: "{w.name}"')

            # try to register permissions
            try:
                w.discoverPermissions(self.workspaceSource)
            except Exception as e:
                traceback.print_exc(file=sys.stdout)
                logManager.error(
                    f'Workspace "{w.name}" unable to discover permissions  ({str(type(e).__name__)}:{e})'
                )

            # try to register monits
            try:
                w.discoverMonits(self.workspaceSource, w)
            except ModuleNotFoundError:
                logManager.info(f'No monits discovered for "{w.name}"')
            except Exception as e:
                traceback.print_exc(file=sys.stdout)
                logManager.error(
                    f'Workspace "{w.name}" unable to discover monits  ({str(type(e).__name__)}:{e})'
                )

            # try to register dataViews
            try:
                w.discoverDataViews(self.workspaceSource)
            except Exception as e:
                traceback.print_exc(file=sys.stdout)
                logManager.error(
                    f'Workspace "{w.name}" unable to discover dataviews  ({str(type(e).__name__)}:{e})'
                )

            workspace_config_section_name = pathlib.Path(
                w.path).resolve().name.upper()

            if workspace_config_section_name in self.config and "disable_jobs" in self.config[
                    workspace_config_section_name] and self.config[
                        workspace_config_section_name]["disable_jobs"]:
                logManager.info(f'Jobs for "{w.name}" are disabled by config')
            else:
                # try to register jobs
                try:
                    w.discoverJobs(self.workspaceSource)
                except Exception as e:
                    traceback.print_exc(file=sys.stdout)
                    logManager.error(
                        f'Workspace "{w.name}" unable to discover jobs  ({str(type(e).__name__)}:{e})'
                    )

            # try to register actions
            try:
                w.discoverActions(self.workspaceSource)
            except Exception as exception:
                traceback.print_exc(file=sys.stdout)
                logManager.error(
                    f'Workspace "{w.name}" unable to discover actions  ({str(type(exception).__name__)}:{exception})'
                )

            # try to register sections
            try:
                w.discoverSections(self.workspaceSource)
            except Exception as exception:
                traceback.print_exc(file=sys.stdout)
                logManager.error(
                    f'Workspace "{w.name}" unable to discover sections  ({str(type(exception).__name__)}:{exception})'
                )

            # try to register node classes
            try:
                w.discoverNodeClasses(self.workspaceSource)
            except ModuleNotFoundError as me:
                logManager.info(f'No node classes discover for "{w.name}"')
            except Exception as exception:
                traceback.print_exc(file=sys.stdout)
                logManager.error(
                    f'Workspace "{w.name}" unable to discover node classes ' +
                    f'({str(type(exception).__name__)}:{exception})')

            # try to register permissions
            try:
                w.discoverPages(self.workspaceSource)
            except Exception as exception:
                traceback.print_exc(file=sys.stdout)
                logManager.error(
                    f'Workspace "{w.name}" unable to discover pages  ({str(type(exception).__name__)}:{exception})'
                )

            # try to register command line commands
            try:
                w.discoverCommands(self.workspaceSource)
            except Exception as exception:
                traceback.print_exc(file=sys.stdout)
                logManager.error(
                    f'Workspace "{w.name}" unable to discover commands  ({str(type(exception).__name__)}:{exception})'
                )

            # try to register widgets
            try:
                w.discoverWidgets(self.workspaceSource)
            except Exception as exception:
                traceback.print_exc(file=sys.stdout)
                logManager.error(
                    f'Workspace "{w.name}" unable to discover widgets ({str(type(exception).__name__)}:{exception})'
                )

            logManager.info("")

    def discover_models(self):
        source = self.workspaceSource
        try:
            imported_source = __import__(source, fromlist=['blah'])
        except Exception as exception:
            logManager.error(f"Unable to locate workspaces {str(exception)}")
            return
        all_current_paths = []

        # all_current_paths.append(imported_source.__path__._path)

        if isinstance(imported_source.__path__, str):
            all_current_paths.append(imported_source.__path__)
        else:
            all_current_paths.extend([x for x in imported_source.__path__])

        from aurori.users.basemodels import UserBase
        
        # remove duplicates
        all_current_paths = list(set(all_current_paths))

        for pkg_path in all_current_paths:
            # Walk through all sub directories
            child_pkgs = [
                p for p in os.listdir(pkg_path)
                if os.path.isdir(os.path.join(pkg_path, p))
            ]

            for child_pkg in child_pkgs:
                try:
                    # fixme: this was assigned to a later unused variable?
                    # imported_package = __import__(source + '.' + child_pkg + '.models', fromlist=['blah'])
                    imported_package = __import__(source + '.' + child_pkg + '.models',
                               fromlist=['blah'])
                    for _, modelpackagename, ispkg in pkgutil.iter_modules(
                            imported_package.__path__,
                            imported_package.__name__ + '.'):
                        workspace_module = __import__(modelpackagename,
                                                      fromlist=['blah'])
                        clsmembers = inspect.getmembers(
                            workspace_module, inspect.isclass)

                        for (_, c) in clsmembers:     
                            if issubclass(c, UserBase) & (c is not UserBase):
                                if self.user_class is not None:
                                    raise Exception("Multiple User models found. For a aurori app only one is allowed.")
                                self.user_class = c
                except ModuleNotFoundError:
                    modelmodule = source + '.' + child_pkg + '.models'
                    logManager.info(f'No model found for {modelmodule}')
                print(child_pkg)
        
        if self.user_class is None:
            logManager.warning(f'No User model derived from aurori.users.basemodels.UserBase found.')
            logManager.warning(f'A aurori app needs a user model to work properly.')
            logManager.warning(f'Please define a derived model named "User" in your workspaces, if needed.')
            logManager.warning(f'The default model will be used, please reference: aurori.users.defaultmodels.User ')
            from aurori.users.defaultmodels import User

    def discover_workspaces(self, source):
        """Recursively walk the supplied package to retrieve all plugins (workspaces)
        """

        try:
            imported_source = __import__(source, fromlist=['blah'])
        except Exception as e:
            logManager.error(f"Unable to locate workspaces {str(e)}")
            return

        all_current_paths = []

        # all_current_paths.append(imported_source.__path__._path)

        if isinstance(imported_source.__path__, str):
            all_current_paths.append(imported_source.__path__)
        else:
            all_current_paths.extend([x for x in imported_source.__path__])

        # remove duplicates
        all_current_paths = list(set(all_current_paths))

        for pkg_path in all_current_paths:
            # Walk through all sub directories
            child_pkgs = [
                p for p in os.listdir(pkg_path)
                if os.path.isdir(os.path.join(pkg_path, p))
            ]

            # Every sub directory contains one workspace
            for child_pkg in child_pkgs:
                imported_package = __import__(source + '.' + child_pkg,
                                              fromlist=['blah'])
                for _, workspacename, ispkg in pkgutil.iter_modules(
                        imported_package.__path__,
                        imported_package.__name__ + '.'):
                    workspaceCounter = 0
                    if not ispkg:
                        workspace_module = __import__(workspacename,
                                                      fromlist=['blah'])
                        clsmembers = inspect.getmembers(
                            workspace_module, inspect.isclass)

                        for (_, c) in clsmembers:
                            # Check for workspace classes
                            if issubclass(c, Workspace) & (c is not Workspace):
                                workspaceCounter += 1
                                if workspaceCounter > 1:
                                    logManager.error(
                                        'Only one workspace is allowed for one folder, other workspaces will skipped'
                                    )
                                    break

                                uri = ""
                                if hasattr(c, 'uri'):
                                    uri = c.uri
                                else:
                                    logManager.error(
                                        f'No uri defined and will not be accessable for workspace: {c.__module__}'
                                    )

                                name = c.__name__
                                if hasattr(c, 'name'):
                                    name = c.name
                                workspaceInstance = c(self.app, self.db, name,
                                                      uri)
                                workspaceInstance.path = os.path.dirname(
                                    workspace_module.__file__)
                                workspace_config_section_name = pathlib.Path(
                                    workspace_module.__file__).parent.resolve(
                                    ).name.upper()
                                logManager.info(
                                    f'Workspace discovered : {workspaceInstance.name} [{c.__module__}]'
                                    + f' with uri "{workspaceInstance.uri}"')

                                if (workspace_config_section_name
                                        in self.config) and (
                                            "disable" in self.config[
                                                workspace_config_section_name]
                                        ) and (self.config[
                                            workspace_config_section_name]
                                               ["disable"]):
                                    logManager.info(
                                        f'Workspace {workspaceInstance.name} [{c.__module__}]'
                                        +
                                        ' is disabled by config and wont show up'
                                    )
                                else:
                                    if workspaceInstance.disable is True:
                                        logManager.info(
                                            f'Workspace {workspaceInstance.name} [{c.__module__}]'
                                            +
                                            ' is disabled by definition and wont show up'
                                        )
                                    else:
                                        self.workspaces.append(
                                            workspaceInstance)



"""
The aurori project

Copyright (C) 2022  Marcus Drobisch,

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

__authors__ = ["Marcus Drobisch"]
__contact__ = "aurori@fabba.space"
__credits__ = []
__license__ = "AGPLv3+"

from aurori import db
from sqlalchemy_utils import ArrowType
import arrow

from aurori.common.jsonDict import JsonDict


class ActionLink(db.Model):
    __tablename__ = 'actionlinks'
    # nonvolatile data stored in the db
    id = db.Column(db.Integer, primary_key=True)
    hash = db.Column(db.String(128), default="")
    workspace = db.Column(db.String(120), default="")
    need_login = db.Column(db.Boolean, default=True)
    action = db.Column(db.String(120), default="")
    action_data_json = db.Column(JsonDict)
    run_only_once = db.Column(db.Boolean, default=True)
    expire_on_date = db.Column(ArrowType, default=arrow.utcnow)
    redirect_to = db.Column(db.String(255), default="")

    def __repr__(self):
        return '<ActionLink {} for {}/{} [expires on {}] >'.format(
            self.link, self.workspace, self.action, self.expire_on_date)



from aurori.actions.action import Action
from aurori.logs import logManager
from aurori.actions import webclientActions


class ProvideMenu(Action):
    def __init__(self, app):
        # logManager.info("ProvideMenu of type Action created")
        super().__init__(app, uri="menu")

    def handle(self, action, user, workspace, actionManager):
        menuBuilder = actionManager.menuBuilder
        logManager.info("Execute registration action")

        # is this really necessary?
        # notification_action = webclientActions.NotificationAction.generate("Menu created", "info")
        notification_action = []
        menu = menuBuilder.buildMenu(user)
        updateMenu_action = webclientActions.UpdateMenuAction.generate(menu)
        return "success", [updateMenu_action, notification_action]



"""
The aurori project

Copyright (C) 2022  Marcus Drobisch,

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

__authors__ = ["Marcus Drobisch"]
__contact__ = "aurori@fabba.space"
__credits__ = []
__license__ = "AGPLv3+"


class BaseAction(object):
    action = 'undefined'
    target = 'undefined'
    source = 'undefined'
    version = 1.0

    def __init__(self, ):
        print("Instance of BaseAction created")

    def execute(self, ):
        print("Execute not defined")

    @classmethod
    def generate(cls, delay=0.0):
        action = {}
        action['action'] = cls.action
        action['target'] = cls.target
        action['version'] = cls.version
        action['source'] = cls.source
        action['delay'] = delay
        return action


from aurori.actions.action import Action
from aurori.logs import logManager
from aurori.actions import webclientActions
from aurori.users import userManager
from aurori.messages import send_mail, send_message
from aurori.actions import generateActionLink


class Register(Action):
    def __init__(self, app):
        # logManager.info("Register of type Action created")
        super().__init__(app, uri="register")

    def handle(self, action, user, workspace, actionManager):
        logManager.info("Execute registration action")
        replyActions = []
        userdata = action["userdata"]
        if userManager.checkUserExist(userdata["email"]):
            replyActions.append(
                webclientActions.NotificationAction.generate(
                    "User already exist", "error"
                )
            )
        else:
            u = userManager.register_user(userdata)
            link = generateActionLink(
                workspace,
                "verifyUser",
                {"email": userdata["email"]},
                "user/login",
                True,
                False,
            )
            data = {
                "username": userdata["firstname"] + " " + userdata["lastname"],
                "action_link": link,
            }
            send_mail(
                [userdata["email"]],
                "Verify your account",
                workspace,
                "requestVerification.mail",
                data,
            )
            send_message(
                u,
                "Welcome",
                workspace,
                "welcome.message",
                data,
                "aurori",
                False,
                "welcome.mail",
            )
            replyActions.append(
                webclientActions.NotificationAction.generate(
                    "User registered", "success"
                )
            )
            replyActions.append(webclientActions.RouteAction.generate("dashboard", 3))

        return "success", replyActions



from typing import Tuple
from aurori.logs import logManager


class FileManager(object):
    """ The FileManager ...
    """
    def __init__(self, ):
        # preparation to instanciate
        self.config = None
        self.app = None
        self.db = None
        self.filestorages = None
        self.default_storage = None

    def init_manager(self, app, db, config):
        self.config = config
        self.app = app
        self.db = db

        logManager.info('Loading filestorages …')

    def get_file(self, uuid_string) -> Tuple[bytes, str, str]:
        from .models import File

        file_object = File.query.filter_by(uuid_string=uuid_string).first()
        if file_object:
            return file_object.file_data, file_object.file_name, file_object.mime_type
        else:
            return None, None, None

    def store_file(self, file, user, filestorage_id=None, workspace=None):
        # fixme: only uploads to the default storage now
        return self.default_storage.store_file(file, user, workspace=workspace)

    def delete_file(self, uuid_string) -> Tuple[str, str]:
        from .models import File

        file_object = File.query.filter_by(uuid_string=uuid_string).first()
        if file_object:
            file_name = file_object.file_name
            if file_object.delete():
                return 'SUCCESS', file_name
            else:
                return 'DELETION FAILED', file_name
        else:
            return 'NOT FOUND', None



"""
The aurori project

Copyright (C) 2022  Marcus Drobisch,

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

__authors__ = ["Marcus Drobisch"]
__contact__ = "aurori@fabba.space"
__credits__ = []
__license__ = "AGPLv3+"

import datetime
from aurori import db


class MonitorEvent(db.Model):
    __tablename__ = 'monitor_event'
    # nonvolatile data stored in the db
    id = db.Column(db.Integer, primary_key=True)
    name = db.Column(db.String(64), default="")
    classifier = db.Column(db.String(64), default=None)
    value = db.Column(db.Float, default=0)
    created_at = db.Column(db.DateTime, default=datetime.datetime.utcnow)
    timestamp = db.Column(db.DateTime, default=datetime.datetime.utcnow)


class MonitorViewItem(db.Model):
    __tablename__ = 'monitor_view_item'
    id = db.Column(db.Integer, primary_key=True)
    monit_key = db.Column(db.String(128), default=None)

    classifier = db.Column(db.String(64), default=None)

    values_last = db.Column(db.Float, default=None)
    values_min = db.Column(db.Float, default=None)
    values_max = db.Column(db.Float, default=None)
    values_average = db.Column(db.Float, default=None)
    values_sum = db.Column(db.Float, default=None)
    values_count = db.Column(db.Integer, default=None)

    total_sum = db.Column(db.Float, default=None)
    total_count = db.Column(db.Integer, default=None)

    timestamp = db.Column(db.DateTime, default=None)
    hour = db.Column(db.Integer, default=None)
    day = db.Column(db.Integer, default=None)
    month = db.Column(db.Integer, default=None)
    year = db.Column(db.Integer, default=None)

    last_event_at = db.Column(db.DateTime, default=None)
    last_reset_at = db.Column(db.DateTime, default=None)
    created_at = db.Column(db.DateTime, default=datetime.datetime.utcnow)



"""
The aurori project

Copyright (C) 2022  Marcus Drobisch,

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

__authors__ = ["Marcus Drobisch"]
__contact__ = "aurori@fabba.space"
__credits__ = []
__license__ = "AGPLv3+"

from flask import Blueprint
from aurori.actions.actionManager import ActionManager

actions_bp = Blueprint('actions', __name__)

actionManager = ActionManager()


def generateActionLink(workspace,
                       action_uri,
                       action_params,
                       redirect_to="",
                       once=True,
                       need_login=True,
                       expire_days=7):
    return actionManager.createActionLink(workspace, action_uri, action_params,
                                          redirect_to, once, need_login,
                                          expire_days)


def executeActionLink(hash, user):
    return actionManager.executeActionLink(hash, user)


from aurori.actions import routes



"""
The aurori project

Copyright (C) 2022  Marcus Drobisch,

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

__authors__ = ["Marcus Drobisch"]
__contact__ = "aurori@fabba.space"
__credits__ = []
__license__ = "AGPLv3+"

import inspect
import pkgutil
import sqlalchemy as sa

from aurori.actions.action import Action
from aurori.nodes.nodeClass import NodeClass
from aurori.workspaces import viewHandler
from aurori.workspaces.dataView import DataView
from aurori.workspaces.page import Page
from aurori.workspaces.permission import Permission
from aurori.jobs.job import Job
from aurori.monits.monits import Monit
from aurori.widgets.widget import Widget
from aurori.jobs import jobManager
from aurori.nodes import nodeManager
from aurori.logs import logManager
from aurori.monits import monit_manager
from aurori.widgets import widgetManager


class Workspace(object):
    """Base class that each workspaceinherit from. 
       The class define methods that all workspaces have to implement
    """

    disable = False

    def __init__(self, app, db, name=None, uri=None):
        self.description = 'UNKNOWN'
        if name is None:
            self.name = type(self).__name__
        else:
            self.name = name
        if uri is None:
            self.uri = self.name
        else:
            self.uri = uri
        self.permissions = {}
        self.dataViews = {}
        self.sections = {}
        self.pages = {}
        self.actions = []
        self.app = app
        self.db = db

    def addSinglePageSection(self, name, page, caption=None, permission=None):
        key = self.name + "_" + name
        if caption is None:
            caption = name
        self.sections[key] = {
            "caption": caption,
            "page": page,
            "permission": permission
        }

    def addPage(self, name, callback, caption=None, permission=None):
        key = self.name + "_" + name
        if caption is None:
            caption = name
        self.pages[key] = {
            "caption": caption,
            "callback": callback,
            "permission": permission
        }

    def addDataView(self, dataView):
        key = dataView.name
        self.dataViews[key] = dataView

    def getPage(self, name):
        key = self.name + "_" + name
        return self.pages[key]

    def discoverJobs(self, workspaceSource):
        jobSource = workspaceSource + '.' + self.name + '.' + 'jobs'
        imported_source = __import__(jobSource, fromlist=['blah'])

        for _, jobname, ispkg in pkgutil.iter_modules(
                imported_source.__path__, imported_source.__name__ + '.'):
            if not ispkg:
                job_module = __import__(jobname, fromlist=['blah'])
                clsmembers = inspect.getmembers(job_module, inspect.isclass)
                for (_, c) in clsmembers:
                    # Check for DataView classes
                    if issubclass(c, Job) & (c is not Job):
                        if c.disable is False:
                            logManager.info(
                                f'Job registered from "{c.__module__}"')
                            jobManager.register_job(self, c, True)

    def discoverMonits(self, workspaceSource, workspace):
        monit_source = workspaceSource + '.' + self.name + '.monits'

        monit_module = __import__(monit_source, fromlist=['blah'])

        clsmembers = inspect.getmembers(monit_module, inspect.isclass)
        for (_, c) in clsmembers:
            # Check for Monit classes
            if issubclass(c, Monit) & (c is not Monit):
                monit_instance = c(workspace)
                monit_instance.define_views()
                logManager.info(
                    f'Monit: "{monit_instance.name}"" loaded from "{c.__module__}"'
                )
                monit_manager.register_monit(monit_instance)

    def discoverPermissions(self, workspaceSource):
        permissionsSource = workspaceSource + '.' + self.name
        imported_source = __import__(permissionsSource, fromlist=['blah'])

        for _, permissionname, ispkg in pkgutil.iter_modules(
                imported_source.__path__, imported_source.__name__ + '.'):
            if not ispkg:
                permission_module = __import__(permissionname,
                                               fromlist=['blah'])
                clsmembers = inspect.getmembers(permission_module,
                                                inspect.isclass)
                for (_, c) in clsmembers:
                    # Check for Permission classes
                    if issubclass(c, Permission) & (c is not Permission):
                        permissionInstance = c()
                        logManager.info(
                            f'Permission: "{permissionInstance.name}"" loaded from "{c.__module__}"'
                        )
                        self.addPermission(str(permissionInstance.name),
                                           permissionInstance)

    def discoverSections(self, workspaceSource):
        '''
        sectionsSource = workspaceSource + '.' + self.name
        imported_source = __import__(sectionsSource, fromlist=['blah'])

        for _, sectionname, ispkg in pkgutil.iter_modules(imported_source.__path__, imported_source.__name__ + '.'):
            if not ispkg:
                section_module = __import__(sectionname, fromlist=['blah'])
                clsmembers = inspect.getmembers(section_module, inspect.isclass)
                for (_, c) in clsmembers:
                    # Check for Section classes
                    if issubclass(c, Section) & (c is not Section):
                        sectionInstance = c()
                        logManager.info(f'Section: "{c.__module__}" created for workspace: "{self.name}"')
                        #self.dataViews[str(dataViewInstance.uri)] = (dataViewInstance)
        '''

    def discoverNodeClasses(self, workspaceSource):
        # for _, node_class, ispkg in pkgutil.iter_modules(imported_source.__path__, imported_source.__name__ + '.'):
        #    if ispkg:
        #        print(node_class)

        node_sources = workspaceSource + '.' + self.name + '.' + 'nodes'
        imported_source = __import__(node_sources, fromlist=['blah'])

        for _, nodename, ispkg in pkgutil.iter_modules(
                imported_source.__path__, imported_source.__name__ + '.'):
            if not ispkg:
                node_module = __import__(nodename, fromlist=['blah'])
                clsmembers = inspect.getmembers(node_module, inspect.isclass)
                for (_, c) in clsmembers:
                    # Check for DataView classes
                    if issubclass(c, NodeClass) & (c is not NodeClass):
                        if c.disable is False:
                            logManager.info(
                                f'NodeClass: "{c.__module__}" loaded from "{node_sources}"'
                            )
                            nodeManager.register_node_class(self, c)
                        else:
                            logManager.info(
                                f'NodeClass: "{c.__module__}" is disabled and wont show up"'
                            )
        '''
        for i in os.listdir(node_sources):
            module_path = os.path.join(node_sources,i)
            if os.path.isdir(module_path):
                module_path_list = module_path.split('/')
                module_path_list.append('node')
                node_module_source = '.'.join(module_path_list)
                node_module = __import__(node_module_source, fromlist=['blah'])
                clsmembers = inspect.getmembers(node_module, inspect.isclass)
                for (_, c) in clsmembers:
                    # Check for DataView classes
                    if issubclass(c, NodeClass) & (c is not NodeClass):
                        if c.disable is False:
                            logManager.info(f'NodeClass: "{c.__module__}" loaded from "{node_sources}"')
                            nodeManager.register_node_class(self, c)
                        else:
                            logManager.info(f'NodeClass: "{c.__module__}" is disabled and wont show up"')
        '''

    def discoverPages(self, workspaceSource):
        pagesSource = workspaceSource + '.' + self.name + '.' + 'pages'
        imported_source = __import__(pagesSource, fromlist=['blah'])

        for _, pagename, ispkg in pkgutil.iter_modules(
                imported_source.__path__, imported_source.__name__ + '.'):
            if not ispkg:
                page_module = __import__(pagename, fromlist=['blah'])
                clsmembers = inspect.getmembers(page_module, inspect.isclass)
                for (_, c) in clsmembers:
                    # Check for DataView classes
                    if issubclass(c, Page) & (c is not Page):
                        pageInstance = c()
                        logManager.info(
                            f'Page: "{pageInstance.name}" loaded from "{c.__module__}"'
                        )
                        if pageInstance.disable is True:
                            logManager.info(
                                f'Page: "{pageInstance.name}" is disabled and wont show up'
                            )
                        else:
                            self.pages[str(pageInstance.name)] = pageInstance

    def discoverDataViews(self, workspaceSource):
        actionsSource = workspaceSource + '.' + self.name + '.' + 'views'
        imported_source = __import__(actionsSource, fromlist=['blah'])

        for _, actionname, ispkg in pkgutil.iter_modules(
                imported_source.__path__, imported_source.__name__ + '.'):
            if not ispkg:
                action_module = __import__(actionname, fromlist=['blah'])
                clsmembers = inspect.getmembers(action_module, inspect.isclass)
                for (_, c) in clsmembers:
                    # Check for DataView classes
                    if issubclass(c, DataView) & (c is not DataView):
                        if hasattr(c, 'name'):
                            dataViewInstance = c(c.name, c.uri)
                            logManager.info(
                                f'DataView "{c.name}" loaded from "{c.__module__}"'
                            )
                        else:
                            dataViewInstance = c(c.__name__, c.uri)
                            logManager.info(
                                f'DataView "{c.__name__}" loaded from "{c.__module__}"'
                            )

                        dataViewInstance.defineProperties()
                        dataViewInstance.defineMetadata()

                        if dataViewInstance.disable is True:
                            logManager.info(
                                f'DataView: "{dataViewInstance.name}" is disabled and wont show up'
                            )
                        else:
                            self.dataViews[str(
                                dataViewInstance.uri)] = (dataViewInstance)

                        if dataViewInstance.entrykey is None:
                            raise LookupError(
                                "DataView {} dont define a key".format(
                                    dataViewInstance.name))

    def discoverActions(self, workspaceSource):

        # create manager wide generic handlers for the view
        self.getViewActionHandler = viewHandler.GetViewActionHandler(
            self.app, self.db)
        self.createViewEntryActionHandler = viewHandler.CreateViewEntryActionHandler(
            self.app, self.db)
        self.removeViewEntryActionHandler = viewHandler.RemoveViewEntryActionHandler(
            self.app, self.db)
        self.updateViewEntryActionHandler = viewHandler.UpdateViewEntryActionHandler(
            self.app, self.db)
        self.executeViewActionsActionHandler = viewHandler.ExecuteViewActionsActionHandler(
            self.app, self.db)

        # add handlers for view actions
        # the view handlers are generic and added to every workspace
        self.actions.append(self.getViewActionHandler)
        self.actions.append(self.createViewEntryActionHandler)
        self.actions.append(self.removeViewEntryActionHandler)
        self.actions.append(self.updateViewEntryActionHandler)
        self.actions.append(self.executeViewActionsActionHandler)

        logManager.info(
            f'Actions for handling DataViews created for workspace "{self.name}"'
        )

        actionsSource = workspaceSource + '.' + self.name + '.' + 'actions'
        # print("Discover actions for", self.uri, "from", actionsSource)
        imported_source = __import__(actionsSource, fromlist=['blah'])

        # print(imported_source)

        for _, actionname, ispkg in pkgutil.iter_modules(
                imported_source.__path__, imported_source.__name__ + '.'):
            if not ispkg:
                action_module = __import__(actionname, fromlist=['blah'])
                clsmembers = inspect.getmembers(action_module, inspect.isclass)
                for (_, c) in clsmembers:
                    # Check for Action classes
                    if issubclass(c, Action) & (c is not Action):
                        actionInstance = c(self.app)
                        logManager.info(
                            f'Action "{c.__module__}" created for workspace "{self.name}"'
                        )
                        if actionInstance.disable is True:
                            logManager.info(
                                f'Action: "{actionInstance.name}" is disabled and wont show up'
                            )
                        else:
                            self.actions.append(actionInstance)

    def discoverCommands(self, workspaceSource):
        commandSource = workspaceSource + '.' + self.name + '.' + 'commands'
        try:
            imported_source = __import__(commandSource, fromlist=['blah'])
            for _, commandname, ispkg in pkgutil.iter_modules(
                    imported_source.__path__, imported_source.__name__ + '.'):
                if not ispkg:
                    try:
                        command_module = __import__(commandname,
                                                    fromlist=['blah'])
                        command_module.register(imported_source.user_cli)
                        logManager.info(
                            'Custom command "{} > {}" registered.'.format(
                                imported_source.user_cli.name,
                                list(imported_source.user_cli.commands)[-1],
                            ))
                    except Exception as e:
                        logManager.error(
                            'Unable to import: {}, exception: {}'.format(
                                commandname, e))

            self.app.cli.add_command(imported_source.user_cli)
        except Exception:
            pass

    def discoverWidgets(self, workspaceSource):
        widgetSource = workspaceSource + '.' + self.name + '.' + 'widgets'
        try:
            imported_source = __import__(widgetSource, fromlist=['blah'])
            for _, widgetname, ispkg in pkgutil.iter_modules(
                    imported_source.__path__, imported_source.__name__ + '.'):
                if not ispkg:
                    widget_module = __import__(widgetname, fromlist=['blah'])
                    clsmembers = inspect.getmembers(widget_module,
                                                    inspect.isclass)
                    for (_, c) in clsmembers:
                        if issubclass(c, Widget) & (c is not Widget):
                            if c.disable is False:
                                logManager.info(
                                    f'Widget registered from "{c.__module__}"')
                                widgetInstance = widgetManager.register_widget(
                                    self, c)
                                self.actions.append(widgetInstance)
                                logManager.info(
                                    f'Widget uri registered from "{c.__module__}"'
                                )
                            else:
                                logManager.info(
                                    f'Widget disabled and not registered from "{c.__module__}"'
                                )

        except Exception:
            pass

    def addPermission(self, name, permission):
        from .models import Permission
        from aurori import db
        key = self.name + "." + name
        description = "No description available"

        if hasattr(permission, 'description'):
            description = permission.description

        engine = db.get_engine()
        table_exists = sa.inspect(engine).has_table(Permission.__tablename__)
        if table_exists:
            p = Permission.query.filter_by(name=key).first()
            if p is None:
                p = Permission(name=key, description=description)
                db.session.add(p)
                db.session.commit()
                p = Permission.query.filter_by(name=key).first()
            else:
                p.caption = description
                db.session.commit()
            self.permissions[key] = p
        else:
            logManager.error(
                'Unable to create permission "{}" for workspace {}'.format(
                    name, self.name))

    def getPermission(self, name):
        key = self.name + "." + name
        return self.permissions[key]

    def createUserHook(self, user):
        pass

    def removeUserHook(self, user):
        pass



"""
The aurori project

Copyright (C) 2022  Marcus Drobisch,

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

__authors__ = ["Marcus Drobisch"]
__contact__ = "aurori@fabba.space"
__credits__ = []
__license__ = "AGPLv3+"

from aurori.logs import logManager
from aurori.actions import webclientActions


class Widget(object):
    group = ''
    name = ''
    description = ''
    disable = False
    required_permission = None
    size = {'w': 0, 'h': 0}  # size in grid units
    position = {'x': 0, 'y': 0}  # distance from top left corner in grid units

    def __init__(self, name=None, uri=None):
        if name is None:
            self.name = type(self).name
        else:
            self.name = name
        if uri is None:
            self.uri = 'widget_{}'.format(self.__class__.__name__.lower())
        else:
            self.uri = uri

    def handle(self, widget, user, workspace, actionManager):
        from aurori.widgets import widgetManager
        self.status = None
        logManager.info('Request widget {} for workspace {} by user {}'.format(
            widget['action'],
            widget['workspace'],
            user,
        ))
        self.response_actions = []
        # check if this widget is registered
        self.registered_widget = widgetManager.get_widget_by_key(
            '{}/{}'.format(widget.workspace, widget.action))
        if self.registered_widget is None:
            notification_action = webclientActions.NotificationAction.generate(
                "Widget not registered", "error")
            self.response_actions.append(notification_action)
            self.status = 'error'
        # check permissions again
        if 'required_permission' in self.registered_widget:
            if not user.admin and not user.has_permission(
                    self.registered_widget['required_permission']):
                notification_action = webclientActions.NotificationAction.generate(
                    "No permission to view this widget.", "error")
                self.response_actions.append(notification_action)
                self.status = 'error'
        # try to parse options
        self.options = None
        if 'options' in widget and widget[
                'options'] is not None and widget['options'] != '':
            self.options = widget['options']



from aurori.actions.action import Action
from aurori.logs import logManager

from workspaces.Users.models import User


class VerifyUser(Action):
    def __init__(self, app):
        # logManager.info("ProvideMenu of type Action created")
        super().__init__(app, uri='verifyUser')

    def handle(self, action, user, workspace, actionManager):
        logManager.info("Request user verification")
        user = User.query.filter_by(email=action.email).first()

        if user is None:
            raise Exception("User not found")

        user.account_verified = True
        return 'success', []

    @staticmethod
    def generate(**kwargs):
        return kwargs



"""
The aurori project

Copyright (C) 2022  Marcus Drobisch,

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

__authors__ = ["Marcus Drobisch"]
__contact__ = "aurori@fabba.space"
__credits__ = []
__license__ = "AGPLv3+"

from aurori.users import auth_bp
from flask import jsonify, request
from pprint import pprint

from flask_jwt_extended import (create_access_token,
                                jwt_refresh_token_required,
                                create_refresh_token, get_jwt_identity,
                                set_access_cookies, set_refresh_cookies,
                                unset_jwt_cookies)


@auth_bp.route('/activate', methods=['POST'])
def activate():
    return "Activate", 200


# With JWT_COOKIE_CSRF_PROTECT set to True, set_access_cookies() and
# set_refresh_cookies() will now also set the non-httponly CSRF cookies
# as well
@auth_bp.route('/login', methods=['POST'])
def login():

    # not used add the moment
    pprint(request.json, indent=4)
    return jsonify({'login': False}), 401
    """ username = request.json.get('username', None)
    password = request.json.get('password', None)

    # Create the tokens we will be sending back to the user
    expires = datetime.timedelta(days=1)
    access_token = create_access_token(identity=username,
                                       expires_delta=expires)
    refresh_token = create_refresh_token(identity=username)

    # Set the JWTs and the CSRF double submit protection cookies
    # in this response
    resp = jsonify({'login': True})
    # the access cookie expire after 15 minutes and have to get refreshed by the client
    set_access_cookies(resp, access_token, max_age=15 * 60)
    # the refresh cookie expire after 30 minutes
    set_refresh_cookies(resp, refresh_token, max_age=30 * 60)
    return resp, 200 """


@auth_bp.route('/login/refresh', methods=['POST'])
@jwt_refresh_token_required
def refresh():
    # Create the new access token
    current_user = get_jwt_identity()
    access_token = create_access_token(identity=current_user)

    # Set the access JWT and CSRF double submit protection cookies
    # in this response
    resp = jsonify({'refresh': True})
    set_access_cookies(resp, access_token)
    return resp, 200


# Because the JWTs are stored in an httponly cookie now, we cannot
# log the user out by simply deleting the cookie in the frontend.
# We need the backend to send us a response to delete the cookies
# in order to logout. unset_jwt_cookies is a helper function to
# do just that.
@auth_bp.route('/logout', methods=['POST'])
def logout():
    resp = jsonify({'logout': True})
    unset_jwt_cookies(resp)
    return resp, 200



from aurori.jobs.job import Job
from datetime import datetime


class CreateTicketJob(Job):

    description = "Creates a ticket in OpenProject"

    def defineArguments(self):
        self.addStringArgument(
            'project',
            label="Project",
            description="Name of OpenProject project to add to")
        self.addStringArgument('subject',
                               label="Subject",
                               description="Subject of the ticket (aka title)")
        self.addStringArgument('object',
                               label="Object",
                               description="Object to return the result to")

    def run(self, **kwargs):
        print('\n Running ticket job')
